/*
 * Project: S.Ha.R.K.
 *
 * Coordinators: 
 *   Giorgio Buttazzo    <giorgio@sssup.it>
 *   Paolo Gai           <pj@gandalf.sssup.it>
 *
 * Authors     : 
 *   Paolo Gai           <pj@gandalf.sssup.it>
 *   (see the web pages for full authors list)
 *
 * ReTiS Lab (Scuola Superiore S.Anna - Pisa - Italy)
 *
 * http://www.sssup.it
 * http://retis.sssup.it
 * http://shark.sssup.it
 */

/*
 * Copyright (C) 2000 Paolo Gai
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

/*
 * CVS :        $Id: initfile.c,v 1.2 2003/03/20 14:59:20 giacomo Exp $
 *
 * File:        $File$
 * Revision:    $Revision: 1.2 $
 * Last update: $Date: 2003/03/20 14:59:20 $
 */

#include "kernel/kern.h"
#include "modules/edf.h"
#include "modules/rr.h"
#include "modules/cbs.h"
#include "modules/dummy.h"

#include "modules/sem.h"
#include "modules/hartport.h"
#include "modules/cabs.h"
#include "modules/pi.h"
#include "modules/pc.h"
#include "modules/srp.h"
#include "modules/npp.h"
#include "modules/nop.h"
#include "modules/nopm.h"

#include "fs/bdevinit.h"
#include "fs/fsinit.h"
#include "fs/bdev.h"

#include "drivers/keyb.h"

/*+ sysyem tick in us +*/
#define TICK 1000

/*+ RR tick in us +*/
#define RRTICK 10000

dev_t root_device=-1;
dev_t temp_device=-1;

int __register_sub_init(void)
{
#if defined(EDFSCHED)
  extern void BD_EDF_register_module(void);
  BD_EDF_register_module();
#elif defined(PSCANSCHED)
  extern void BD_PSCAN_register_module(void);
  BD_PSCAN_register_module();  
#endif
  return 0;
}

int choose_root_callback(dev_t dev,u_int8_t fs)
{
  if (fs==FS_MSDOS) return dev;
  return -1;
}

int choose_temp_callback(__dev_t dev,__uint8_t fs)
{
  static int flag=0;
  if (fs==FS_MSDOS) {
    if (flag) return dev;
    flag=1;
  }
  return -1;
}

int __bdev_sub_init(void)
{
  BDEV_PARMS bdev=BASE_BDEV;
  
  bdev_def_showinfo(bdev,FALSE);
  bdev_init(&bdev);

  root_device=bdev_scan_devices(choose_root_callback);
  if (root_device<0) {
    cprintf("can't find root device to mount on /!!!\n");
    sys_end();
    return -1;
  }

  return 0;

}

int __fs_sub_init(void)
{
  extern int libc_initialize(void);
  FILESYSTEM_PARMS fs=BASE_FILESYSTEM;
  
  filesystem_def_rootdevice(fs,root_device);
  filesystem_def_fs(fs,FS_MSDOS);
  filesystem_def_showinfo(fs,FALSE);
  filesystem_init(&fs);

  libc_initialize();

  return 0;
}

TIME __kernel_register_levels__(void *arg)
{
  struct multiboot_info *mb = (struct multiboot_info *)arg;
  extern int __register_sub_init(void);

  EDF_register_level(EDF_ENABLE_ALL);
  RR_register_level(RRTICK, RR_MAIN_YES, mb);
  CBS_register_level(CBS_ENABLE_ALL, 0);
  dummy_register_level();

  SEM_register_module();

  CABS_register_module();

  PI_register_module();
  PC_register_module();
  NPP_register_module();
  SRP_register_module();
  NOP_register_module();
  NOPM_register_module();

  __register_sub_init();
  
  return TICK;
}

TASK __init__(void *arg)
{
  struct multiboot_info *mb = (struct multiboot_info *)arg;
  KEYB_PARMS	keyb = BASE_KEYB;
  extern int __bdev_sub_init(void);
  extern int __fs_sub_init(void);
  
  HARTPORT_init();
  
  KEYB_init(&keyb);

  __bdev_sub_init();
  __fs_sub_init();
  
  __call_main__(mb);

  return (void *)0;
}

